using System;
using System.Data;
using System.Data.SqlClient;
using System.Collections;

namespace k.App.Data
{
    /// <summary>
    /// Summary description for DB.
    /// </summary>
    public class DB : IDisposable
    {
        public DB()
        {
            this.ExecutionTimeout = Settings.ExecutionTimeout;
        }

        public DB(string ConnectionString)
        {
            Settings.ExternalServerConnString = ConnectionString;
        }

        #region Properties
        /// <summary>
        /// SP Execution Timeout
        /// </summary>
        private int m_ExecutionTimeout = 30;
        public int ExecutionTimeout
        {
            get { return this.m_ExecutionTimeout; }
            set { this.m_ExecutionTimeout = value; }
        }

        /// <summary>
        /// Transaction' tutan private deiken
        /// </summary>
        private System.Data.SqlClient.SqlTransaction m_Transaction = null;
        public bool InTransaction
        {
            get { return (this.m_Transaction != null); }
        }
        /// <summary>
        /// Transaction' balatr.
        /// </summary>
        public void BeginTransaction()
        {
            setSqlConnection("", DatabaseDirection.TransactionDatabase);
            this.BeginTransaction(IsolationLevel.ReadCommitted);
        }
        /// <summary>
        /// Connection' tutan private deiken
        /// </summary>
        private System.Data.SqlClient.SqlConnection m_Connection = null;
        /// <summary>
        /// Connection  Hangi Server yapmas gerektiini tutar.
        /// </summary>
        private DatabaseDirection m_ExecutionDb = 0;
        private DatabaseDirection ExecutionDb
        {
            get { return m_ExecutionDb; }
            set { m_ExecutionDb = value; }
        }
        #endregion

        #region Methods
        /// <summary>
        /// Transaction Balatr
        /// </summary>
        /// <param name="isolationLevel">Transaction Isolation Level</param>
        public void BeginTransaction(IsolationLevel isolationLevel)
        {
            if (this.m_Connection != null)
            {
                if (!this.InTransaction)
                {
                    if (this.m_Connection.State == ConnectionState.Closed)
                    {
                        this.m_Connection.Open();
                    }
                    this.m_Transaction = this.m_Connection.BeginTransaction(isolationLevel);
                }
                else
                {
                    throw new TransactionAlreadyExists("Ak bir transaction zerine yeni bir transaction aamazsnz..!");
                }
            }

        }
        /// <summary>
        /// Transaction' commit eder.
        /// </summary>
        public void CommitTransaction()
        {
            if (this.InTransaction)
            {
                this.m_Transaction.Commit();
                this.m_Transaction.Dispose();
                this.m_Transaction = null;
                this.CloseDbConnection();
            }
            else
            {
                throw new NoTransactionException("Commit edilecek ak bir transaction bulunamad..!");
            }
        }
        /// <summary>
        /// Transaction' Rollback eder.
        /// </summary>
        public void RollbackTransaction()
        {
            if (this.InTransaction)
            {
                this.m_Transaction.Rollback();
                this.m_Transaction.Dispose();
                this.m_Transaction = null;
                this.CloseDbConnection();
            }
            else
            {
                throw new NoTransactionException("Rollback edilecek ak bir transaction bulunamad..!");
            }
        }
        /// <summary>
        /// Transaction' Command'a bind eder.
        /// </summary>
        /// <param name="cmd"></param>
        private void BindConnectionContext(IDbCommand cmd)
        {
            cmd.Connection = this.m_Connection;
            if (this.InTransaction)
            {
                if (cmd != null)
                {
                    cmd.Transaction = this.m_Transaction;
                }
            }
        }

        /// <summary>
        /// Internal SqlConnection' set Eder.
        /// </summary>
        private void setSqlConnection(string spName, DatabaseDirection dbDirection)
        {
            if (InTransaction && dbDirection == DatabaseDirection.ReportingDatabase)
                throw new Exception("Reporting Server zerinde transaction balatamassnz.");

            if (dbDirection == DatabaseDirection.ReportingDatabase && spName.IndexOf("_sel_") == -1)
                throw new Exception("Reporting Server zerinde 'Select' sorgusu haricinde Store Procedure altrlamaz.");

            if (!InTransaction)
            {
                switch (dbDirection)
                {
                    case DatabaseDirection.TransactionDatabase:
                        this.m_Connection = new System.Data.SqlClient.SqlConnection(Settings.AppServerConnString);
                        break;
                    case DatabaseDirection.ReportingDatabase:
                        this.m_Connection = new System.Data.SqlClient.SqlConnection(Settings.RptServerConnString);
                        break;
                    case DatabaseDirection.ExternalDatabase:
                        this.m_Connection = new System.Data.SqlClient.SqlConnection(Settings.ExternalServerConnString);
                        break;
                }
            }
        }
        /// <summary>
        /// Internal Connection' aar
        /// </summary>
        private void OpenDbConnection()
        {
            if (this.m_Connection != null)
            {
                if (this.m_Connection.State == ConnectionState.Closed)
                {
                    this.m_Connection.Open();
                }
            }
        }
        /// <summary>
        /// Database connection'n kapatr. Ak transaction varsa rollback eder.
        /// </summary>
        public void CloseDbConnection()
        {

            if (this.m_Connection != null)
            {
                if (this.m_Connection.State != ConnectionState.Closed)
                {
                    if (this.InTransaction)
                    {
                        this.RollbackTransaction();
                    }
                    this.CloseAllDbLevelTransactions();
                    this.m_Connection.Close();
                }
            }
        }

        /// <summary>
        /// Connections zernideki tm connectionlar kapar.
        /// </summary>
        public void CloseAllDbLevelTransactions()
        {
            try
            {
                System.Data.SqlClient.SqlCommand cmd = new System.Data.SqlClient.SqlCommand();
                cmd.CommandType = CommandType.Text;
                cmd.CommandTimeout = this.ExecutionTimeout;
                cmd.Connection = this.m_Connection;
                cmd.CommandText = "IF @@TRANCOUNT > 0 ROLLBACK TRAN;";
                cmd.ExecuteNonQuery();
            }
            catch (Exception exp)
            {
                throw exp;
            }
        }
        #endregion

        #region Execution Commands
        /// <summary>
        /// Geriye deer (output parametresi hari)  dndrmeyecek SP'ler iin kullanlr
        /// </summary>
        /// <param name="spName">SP Ad</param>
        /// <param name="parameters">Sp Parametreleri</param>
        public void ExecuteNonQuery(DatabaseDirection ExecutionDb, string spName, params SqlParameter[] parameters)
        {
            System.Data.SqlClient.SqlCommand cmd = null;
            try
            {

                setSqlConnection(spName, ExecutionDb);
                cmd = new System.Data.SqlClient.SqlCommand(spName);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.CommandTimeout = this.ExecutionTimeout;
                if (parameters != null)
                {
                    for (int i = 0; i < parameters.Length; i++)
                    {
                        cmd.Parameters.Add(parameters[i]);
                    }
                }

                this.OpenDbConnection();
                this.BindConnectionContext(cmd);
                cmd.ExecuteNonQuery();
                if (!this.InTransaction)
                {
                    this.CloseDbConnection();
                }
                cmd.Parameters.Clear();
            }
            catch (System.Exception exc)
            {
                try
                {
                    if (!this.InTransaction)
                    {
                        this.CloseDbConnection();
                    }
                    cmd.Parameters.Clear();
                }
                catch { }

                throw (exc);
            }
        }
        /// <summary>
        /// Sp altrlmas sonucunda geriye sadece tek bir deer dnecekse kullanlr. 
        /// Output parametresi olarak ekstra deerler geriye dnebiliri
        /// </summary>
        /// <param name="spName">Sp Ad</param>
        /// <param name="parameters">Sp Parametreleri</param>
        /// <returns></returns>
        public object ExecuteScalar(DatabaseDirection ExecutionDb, string spName, params SqlParameter[] parameters)
        {
            System.Data.SqlClient.SqlCommand cmd = null;
            try
            {
                setSqlConnection(spName, ExecutionDb);
                cmd = new System.Data.SqlClient.SqlCommand(spName);
                cmd.CommandTimeout = this.ExecutionTimeout;
                cmd.CommandType = CommandType.StoredProcedure;
                if (parameters != null)
                {
                    for (int i = 0; i < parameters.Length; i++)
                    {
                        cmd.Parameters.Add(parameters[i]);
                    }
                }
                this.OpenDbConnection();
                this.BindConnectionContext(cmd);
                object retval = cmd.ExecuteScalar();
                if (!this.InTransaction)
                {
                    this.CloseDbConnection();
                }
                cmd.Parameters.Clear();
                return retval;
            }
            catch (Exception exc)
            {
                try
                {
                    if (!this.InTransaction)
                    {
                        this.CloseDbConnection();
                    }
                    cmd.Parameters.Clear();
                }
                catch { }
                throw (exc);
            }
        }
        /// <summary>
        /// Sp geriye birden fazla resultset dnyorsa kullanlr
        /// </summary>
        /// <param name="spName">Sp Ad</param>
        /// <param name="parameters">Sp Parametreleri</param>
        /// <returns>DataSet</returns>
        public DataSet ExecuteDataSet(DatabaseDirection ExecutionDb, string spName, params SqlParameter[] parameters)
        {
            DataSet ds = new DataSet();
            System.Data.SqlClient.SqlCommand cmd = null;
            try
            {
                setSqlConnection(spName, ExecutionDb);
                cmd = new System.Data.SqlClient.SqlCommand(spName);
                cmd.CommandTimeout = this.ExecutionTimeout;
                cmd.CommandType = CommandType.StoredProcedure;
                if (parameters != null)
                {
                    for (int i = 0; i < parameters.Length; i++)
                    {
                        cmd.Parameters.Add(parameters[i]);
                    }
                }
                this.OpenDbConnection();
                this.BindConnectionContext(cmd);
                System.Data.SqlClient.SqlDataReader rdr = cmd.ExecuteReader();
                DataTable dt = null;
                int tableIndex = 0;
                try
                {
                    do
                    {
                        Hashtable ColumnsAdded = new Hashtable();
                        dt = new DataTable(string.Format("Table{0}", ++tableIndex));
                        int z = rdr.FieldCount;
                        for (int i = 0; i < z; i++)
                        {
                            int NewColumnIndex = 0;
                            string ColumnName = rdr.GetName(i);
                            if (dt.Columns.Contains(ColumnName))
                            {
                                NewColumnIndex = Convert.ToInt32(ColumnsAdded[ColumnName]);
                                ColumnName += "_" + NewColumnIndex.ToString();
                                ColumnsAdded[ColumnName] = NewColumnIndex++;
                            }
                            dt.Columns.Add(new DataColumn(ColumnName, rdr.GetFieldType(i)));
                        }
                        int colCount = dt.Columns.Count;
                        while (rdr.Read())
                        {
                            object[] arr = new object[colCount];
                            rdr.GetValues(arr);
                            dt.Rows.Add(arr);
                        }
                        ds.Tables.Add(dt);
                    } while (rdr.NextResult());
                    rdr.Close();
                }
                catch (Exception)
                {
                    rdr.Close();
                    throw;
                }
                if (!this.InTransaction)
                {
                    this.CloseDbConnection();
                }
                return ds;
            }
            catch (Exception exc)
            {
                try
                {
                    if (!this.InTransaction)
                        this.CloseDbConnection();
                }
                catch { }

                throw (exc);
            }
        }

        /// <summary>
        /// Sp geriye tek bir resultset dnyorsa kullanlr
        /// </summary>
        /// <param name="spName">Sp Ad</param>
        /// <param name="parameters">Sp Parametreleri</param>
        /// <returns>DataTable</returns>
        public DataTable ExecuteDataTable(DatabaseDirection ExecutionDb, string spName, params SqlParameter[] parameters)
        {
            try
            {
                setSqlConnection(spName, ExecutionDb);
                System.Data.SqlClient.SqlCommand cmd = new System.Data.SqlClient.SqlCommand(spName);
                cmd.CommandTimeout = this.ExecutionTimeout;
                cmd.CommandType = CommandType.StoredProcedure;
                if (parameters != null)
                {
                    for (int i = 0; i < parameters.Length; i++)
                    {
                        cmd.Parameters.Add(parameters[i]);
                    }
                }
                this.OpenDbConnection();
                this.BindConnectionContext(cmd);
                System.Data.SqlClient.SqlDataReader rdr = cmd.ExecuteReader();
                DataTable dt = null;
                try
                {
                    Hashtable ColumnsAdded = new Hashtable();
                    dt = new DataTable();
                    int z = rdr.FieldCount;
                    for (int i = 0; i < z; i++)
                    {
                        int NewColumnIndex = 0;
                        string ColumnName = rdr.GetName(i);
                        if (dt.Columns.Contains(ColumnName))
                        {
                            NewColumnIndex = Convert.ToInt32(ColumnsAdded[ColumnName]);
                            ColumnName += "_" + NewColumnIndex.ToString();
                            ColumnsAdded[ColumnName] = NewColumnIndex++;
                        }
                        dt.Columns.Add(new DataColumn(ColumnName, rdr.GetFieldType(i)));
                    }
                    int colCount = dt.Columns.Count;
                    while (rdr.Read())
                    {
                        object[] arr = new object[colCount];
                        rdr.GetValues(arr);
                        dt.Rows.Add(arr);
                    }
                    rdr.Close();
                }
                catch (Exception)
                {
                    rdr.Close();
                    throw;
                }
                if (!this.InTransaction)
                {
                    this.CloseDbConnection();
                }
                cmd.Parameters.Clear();
                return dt;
            }
            catch (Exception exc)
            {
                try
                {
                    if (!this.InTransaction)
                        this.CloseDbConnection();
                }
                catch { }
                throw (exc);
            }
        }
        /// <summary>
        /// Alan SqlReader'lar tutan private deiken
        /// </summary>
        private System.Collections.ArrayList arr = new System.Collections.ArrayList();
        /// <summary>
        /// SqlDataReader geriye dner
        /// </summary>
        /// <param name="spName">Sp Ad</param>
        /// <param name="parameters">Sp Parametreleri</param>
        /// <returns>SqlDataReader</returns>
        public System.Data.SqlClient.SqlDataReader ExecuteReader(DatabaseDirection ExecutionDb, string spName, params SqlParameter[] parameters)
        {
            try
            {
                setSqlConnection(spName, ExecutionDb);
                System.Data.SqlClient.SqlCommand cmd = new System.Data.SqlClient.SqlCommand(spName);
                cmd.CommandTimeout = this.ExecutionTimeout;
                cmd.CommandType = CommandType.StoredProcedure;
                if (parameters != null)
                {
                    for (int i = 0; i < parameters.Length; i++)
                    {
                        cmd.Parameters.Add(parameters[i]);
                    }
                }
                this.OpenDbConnection();
                this.BindConnectionContext(cmd);
                System.Data.SqlClient.SqlDataReader rdr = cmd.ExecuteReader();
                this.arr.Add(rdr);
                return rdr;
            }
            catch (Exception exc)
            {
                try
                {
                    if (!this.InTransaction)
                        this.CloseDbConnection();
                }
                catch { }
                throw (exc);
            }
        }
        /// <summary>
        /// Verilen objenin database'deki ID'sini dner
        /// </summary>
        /// <param name="objectName">object ad</param>
        /// <param name="xType">Object tipi</param>
        /// <returns>Object Id (long)</returns>
        public long GetObjectId(DatabaseDirection ExecutionDb, string objectName, string xType)
        {
            setSqlConnection("", ExecutionDb);
            System.Data.SqlClient.SqlCommand cmd = new System.Data.SqlClient.SqlCommand("Select id from sysobjects where name='" + objectName + "' and xtype='" + xType + "'");
            this.OpenDbConnection();
            this.BindConnectionContext(cmd);
            object retval = cmd.ExecuteScalar();
            if (!this.InTransaction)
            {
                this.CloseDbConnection();
            }
            if (retval == null)
            {
                return -1;
            }
            else
            {
                return Convert.ToInt64(retval);
            }
        }
        /// <summary>
        /// Verilen bir SQL Cmleciini DataSet'e doldurur
        /// </summary>
        /// <param name="commandText">Sql Cmlecii</param>
        /// <returns>DataSet</returns>
        internal DataSet ExecuteCommandToDataSet(DatabaseDirection ExecutionDb, string commandText)
        {
            setSqlConnection(commandText, ExecutionDb);
            System.Data.SqlClient.SqlCommand cmd = new System.Data.SqlClient.SqlCommand(commandText);
            cmd.CommandTimeout = this.ExecutionTimeout;
            this.OpenDbConnection();
            this.BindConnectionContext(cmd);
            System.Data.SqlClient.SqlDataAdapter adp = new System.Data.SqlClient.SqlDataAdapter(cmd);
            DataSet ds = new DataSet();
            adp.Fill(ds);
            if (!this.InTransaction)
            {
                this.CloseDbConnection();
            }
            return ds;
        }
        /// <summary>
        /// Verilen Sql cmleciini DataTable'a doldurur
        /// </summary>
        /// <param name="commandText">Sql Cmlecii</param>
        /// <returns>DataTable</returns>
        internal DataTable ExecuteCommandToDataTable(DatabaseDirection ExecutionDb, string commandText)
        {
            DataSet ds = this.ExecuteCommandToDataSet(ExecutionDb, commandText);
            DataTable dt = ds.Tables[0];
            ds.Tables.Remove(dt);
            return dt;
        }
        /// <summary>
        /// Verilen SQL Cmleciini altrr
        /// </summary>
        /// <param name="commandText">Sql Cmlecii</param>
        internal void ExecuteCommand(DatabaseDirection ExecutionDb, string commandText)
        {
            setSqlConnection(commandText, ExecutionDb);
            System.Data.SqlClient.SqlCommand cmd = new System.Data.SqlClient.SqlCommand(commandText);
            cmd.CommandTimeout = this.ExecutionTimeout;
            this.OpenDbConnection();
            this.BindConnectionContext(cmd);
            try
            {
                cmd.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw ex;
            }
            if (!this.InTransaction)
            {
                this.CloseDbConnection();
            }
        }
        #endregion

        ~DB()
        {
            Dispose();
        }

        #region IDisposable Members

        public void Dispose()
        {
            if (m_Connection.State == ConnectionState.Open)
                m_Connection.Close();

            m_Connection.Dispose();

            GC.SuppressFinalize(this);
        }

        #endregion
    }
}
